/*
 * file: pv_string.c
 * This file is part of the libpv library.
 *
 * Copyright 2005 Pengutronix
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 */

#include <libpv_config.h>

#include <errno.h>
#include <string.h>

#include <libpv.h>
#include <pv_types.h>
#include <pv_bs.h>

int pv_cmp_string(pv_engine_t	*engine_anon,
		  const char	*pv_string,
		  char		*s2)
{
	struct pv_engine	*engine = (struct pv_engine *)engine_anon;
	struct pv		*pv;
	int			finished = 0;

	if (!engine || !engine->pv_list)
		return INT32_MIN;

	for (goto_head(&engine->pv_list); !finished; goto_next(&engine->pv_list)) {
		
		if (is_last(engine->pv_list))
			finished=1;

		pv = (struct pv *)list_get_element_data(engine->pv_list);
		if (engine->store.map + pv->offset == pv_string) {
			if (pv->type != PV_TYPE_STRING)
				return -EINVAL;
			return strncmp(pv_string, s2, pv->size);
		}
	}

	return INT32_MIN;
}


int pv_get_string(pv_engine_t	*engine_anon,
		  const char	*pv_string,
		  char		*dest,
		  size_t	n)
{
	struct pv_engine	*engine = (struct pv_engine *)engine_anon;
	struct pv		*pv;
	int			finished = 0;
	size_t			size;

	if (!engine || !engine->pv_list)
		return -EINVAL;

	for (goto_head(&engine->pv_list); !finished; goto_next(&engine->pv_list)) {
		
		if (is_last(engine->pv_list))
			finished=1;

		pv = (struct pv *)list_get_element_data(engine->pv_list);
		if (engine->store.map + pv->offset == pv_string) {
			if (pv->type != PV_TYPE_STRING)
				return -EINVAL;
			size = MIN(pv->size - 1, n);
			strncpy(dest, pv_string, size);
			dest[size] = 0;
			return 0;
		}
	}

	return -EINVAL;
}


/*
 * this functions flushes!
 */
int pv_set_string(pv_engine_t	*engine_anon,
		  char		*pv_string,
		  char		*src)
{
	struct pv_engine	*engine = (struct pv_engine *)engine_anon;
	struct pv		*pv;
	int			finished = 0;

	if (!engine || !engine->pv_list)
		return -EINVAL;

	for (goto_head(&engine->pv_list); !finished; goto_next(&engine->pv_list)) {
		
		if (is_last(engine->pv_list))
			finished=1;

		pv = (struct pv *)list_get_element_data(engine->pv_list);
		if (engine->store.map + pv->offset == pv_string) {
			if (pv->type != PV_TYPE_STRING)
				return -EINVAL;
			strncpy(pv_string, src, pv->size);
			pv_string[pv->size - 1] = 0;

			pv_bs_flush_pv(engine, pv);
			return 0;
		}
	}

	return -EINVAL;
}

/* end of file pv_string.c */
