#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <math.h>

#include <libpv.h>

static char *prompt="test";
static int shutdown = 0;

#define	PV_ARRAY_LEN	(32)

void signalhandler (int sig)
{
	if (sig == SIGPIPE) return;
	if (sig == SIGHUP) {
		fprintf(stderr, "%s SIGHUP received\n", prompt);
	}
	if (sig == SIGTERM || sig == SIGINT) {
		fprintf(stderr, "%s received SIGTERM or SIGINT, shutting down\n", prompt);
		shutdown = 1;
	}
}

int main(int argc, char *argv[])
{
	struct sigaction	action;
	pv_engine_t 		pv_engine;
	pv_engine_attr_t	attr;
	uint32_t		*pv_array, *pv_f, *pv_a, *pv_s;
	int			err;
	int			i;
	float			x = 0;

	err = pv_engine_attr_init_from_cmdline(&attr, argc, argv);
	if (err)
		exit (EXIT_FAILURE);
	err = pv_init(&pv_engine, &attr);
	if (err)
		exit (EXIT_FAILURE);

	printf("%s: engine created\n", prompt);

	/* setup signal handler */
	action.sa_handler = signalhandler;
	sigemptyset (&action.sa_mask);
	action.sa_flags = 0;
	sigaction (SIGTERM, &action, 0);	/* catches kill from the shell   */
	sigaction (SIGINT,  &action, 0);	/* catches Ctrl-C from the shell */
	sigaction (SIGPIPE, &action, 0);	/* catches Broken Pipe exception */
	sigaction (SIGHUP,  &action, 0);	/* catches HUP                   */

	pv_array = pv_register_uint32_array(&pv_engine, "array");
	pv_s = pv_register_uint32(&pv_engine, "scale");
	pv_f = pv_register_uint32(&pv_engine, "freq");
	pv_a = pv_register_uint32(&pv_engine, "amp");

	pv_default(&pv_engine, pv_f);
	pv_default(&pv_engine, pv_a);

	while (!shutdown) {
		pv_lock(&pv_engine);
		for (i = 0; i < PV_ARRAY_LEN; i++) {
			pv_array[i] = sin(M_PI * i * (*pv_s / 10.0) / PV_ARRAY_LEN + x ) * *pv_a;
		}
		//pv_flush(&pv_engine, pv_array, PV_ARRAY_LEN);
		pv_unlock(&pv_engine);
		
		x += *pv_f * PV_ARRAY_LEN / M_PI / 400;
		
		//printf("%s: plotArray[10]=%i\n", prompt, pv_array[10]);

		usleep(100 * 1000);
	}
	
	pv_destroy(&pv_engine);
	
	return EXIT_SUCCESS;
}

